@@ -40,8 +40,9 @@ This module provides different methods to access and filter the statistics compi
 
 =head2 new()
 
-Create a new C<Sys::Statistics::Linux::Compilation> object. This creator is only useful if you
-don't call C<get()> of C<Sys::Statistics::Linux>. You can create a new object with:
+Create a new C<Compilation> object. This creator is only useful if you
+don't call C<get()> of C<Sys::Statistics::Linux>. You can create a new
+object with:
 
     my $lxs  = Sys::Statistics::Linux::LoadAVG->new();
     my $load = $lxs->get;
@@ -53,6 +54,8 @@ don't call C<get()> of C<Sys::Statistics::Linux>. You can create a new object wi
 
 =item sysinfo()
 
+=item cpuinfo()
+
 =item cpustats()
 
 =item procstats()
@@ -63,10 +66,6 @@ don't call C<get()> of C<Sys::Statistics::Linux>. You can create a new object wi
 
 =item netstats()
 
-=item netinfo()
-
-C<netinfo()> provides raw data - no deltas.
-
 =item sockstats()
 
 =item diskstats()
@@ -88,14 +87,19 @@ returns the first level keys of the statistics. Example:
     my @dev  = $stat->netstats;                 # the devices eth0, eth1, ...
     my $eth0 = $stat->netstats('eth0');         # eth0 statistics as a hash reference
     my @keys = $stat->netstats('eth0');         # the statistic keys
-    my @vals = $stat->netstats('eth0', @keys);  # the values for the passed device and @keys
-    my $val  = $stat->netstats('eth0', $key);   # the value for the passed device and key
+    my @vals = $stat->netstats('eth0', @keys);  # the values for @keys
 
-Sorted ...
+I was thinking about to return all keys sorted but if you need that you can call
 
     my @dev  = sort $stat->netstats;
     my @keys = sort $stat->netstats('eth0');
 
+LoadAVG example:
+
+    my $load = $stat->loadavg;                  # loadavg as a hash reference
+    my @keys = $stat->loadavg;                  # the statistic keys
+    my @vals = $stat->loadavg(@keys);           # the values for @keys
+
 =head2 pstop()
 
 This method is looking for top processes and returns a sorted list of PIDs as an array or
@@ -117,8 +121,9 @@ If you want to get all processes:
 =head2 search(), psfind()
 
 Both methods provides a simple scan engine to find special statistics. Both methods except a filter
-as a hash reference. It's possible to pass the statistics as second argument if the data is not stored
-in the object.
+as a hash reference as the first argument. If your data comes from extern - maybe from a client that
+send his statistics to the server - you can set the statistics as the second argument. The second
+argument have to be a hash reference as well.
 
 The method C<search()> scans for statistics and rebuilds the hash tree until that keys that matched
 your filter and returns the hits as a hash reference.
@@ -147,11 +152,41 @@ This would return the following matches:
     * all cpu where "iowait" is grather than 10
     * only disk '/dev/sda1' if "usageper" is grather than 80
 
+If the statistics are not gathered by the current process then you can handoff statistics as an
+argument.
+
+    my %stats = (
+        cpustats => {
+            cpu => {
+                system => '51.00',
+                total  => '51.00',
+                idle   => '49.00',
+                nice   => '0.00',
+                user   => '0.00',
+                iowait => '0.00'
+            }
+        }
+    );
+
+    my $stat = Sys::Statistics::Linux::Compilation(\%stats);
+
+    my %filter = (
+        cpustats => {
+            total => 'gt:50'
+        }
+    );
+
+    my $hits = $stat->search(\%filter);
+
 The method C<psfind()> scans for processes only and returns a array reference with all process
 IDs that matched the filter. Example:
 
     my $pids = $stat->psfind({ cmd => qr/init/, owner => 'eq:apache' });
 
+You can handoff the statistics as second argument as well.
+
+    my $pids = $stat->psfind(\%filter, \%stats);
+
 This would return the following process ids:
 
     * processes that matched the command "init"
@@ -193,7 +228,7 @@ Please report all bugs to <jschulz.cpan(at)bloonix.de>.
 
 Jonny Schulz <jschulz.cpan(at)bloonix.de>.
 
-Thanks to Moritz Lenz for his suggestion for the name of this module.
+Thanks to Moritz Lenz for his suggestion for the name for this module.
 
 =head1 COPYRIGHT
 
@@ -204,16 +239,15 @@ This program is free software; you can redistribute it and/or modify it under th
 =cut
 
 package Sys::Statistics::Linux::Compilation;
+our $VERSION = '0.01';
 
 use strict;
 use warnings;
 use Carp qw(croak);
 
-our $VERSION = '0.10';
-
 # Creating the statistics accessors
 BEGIN {
-    foreach my $stat (qw/sysinfo procstats memstats sockstats loadavg filestats/) {
+    foreach my $stat (qw/sysinfo cpuinfo procstats memstats sockstats loadavg filestats/) {
         no strict 'refs';
         *{$stat} = sub {
             use strict 'refs';
@@ -225,7 +259,7 @@ BEGIN {
             return wantarray ? keys %{$self->{$stat}} : $self->{$stat};
         };
     }
-    foreach my $stat (qw/cpustats pgswstats netstats netinfo diskstats diskusage processes/) {
+    foreach my $stat (qw/cpustats pgswstats netstats diskstats diskusage processes/) {
         no strict 'refs';
         *{$stat} = sub {
             use strict 'refs';
@@ -248,7 +282,7 @@ BEGIN {
 sub new {
     my ($class, $stats) = @_;
     unless (ref($stats) eq 'HASH') {
-        croak 'Usage: $class->new( \%statistics )';
+        croak 'Usage: class->new( \%statistics )';
     }
     return bless $stats, $class;
 }
@@ -276,7 +310,24 @@ sub search {
         # matched the searched string
 
         foreach my $x (keys %{$fref}) {
+
+            # if $fref->{$x} is a hash ref then the next key have to
+            # match the statistic key. this is used for statistics
+            # like NetStats or Processes that uses a hash key for the
+            # device name or process id. NetStats example... if
+            #
+            #    $fref->{eth0}->{ttbyt}
+            #
+            # is defined as a filter then the key "eth0" have to match
+            #
+            #    $proc->{eth0}
+            #
+            # then we look if "ttbyt" matched the searched statistic name
+            # and compare the values. if the comparing returns TRUE we the
+            # hash is copied until the matched key-value pair.
+
             if (ref($fref->{$x}) eq 'HASH') {
+
                 # if the key $proc->{eth0} doesn't exists
                 # then we continue with the next defined filter
                 next unless exists $proc->{$x};
@@ -291,13 +342,7 @@ sub search {
                 foreach my $key (keys %{$proc}) {
                     if (ref($proc->{$key}) eq 'HASH') {
                         $subref = $proc->{$key};
-                        if (ref $subref->{$x} eq 'HASH') {
-                            foreach my $y (keys %{$subref->{$x}}) {
-                                if ($self->_compare($subref->{$x}->{$y}, $fref->{$x})) {
-                                    $hits{$opt}{$key}{$x}{$y} = $subref->{$x}->{$y};
-                                }
-                            }
-                        } elsif (defined $subref->{$x} && $self->_compare($subref->{$x}, $fref->{$x})) {
+                        if (defined $subref->{$x} && $self->_compare($subref->{$x}, $fref->{$x})) {
                             $hits{$opt}{$key}{$x} = $subref->{$x};
                         }
                     } else { # must be a scalar now
@@ -310,7 +355,6 @@ sub search {
             }
         }
     }
-
     return wantarray ? %hits : \%hits;
 }
 
@@ -320,21 +364,11 @@ sub psfind {
     my $proc   = $self->{processes} or return undef;
     my @hits   = ();
 
-    PID: foreach my $pid (keys %{$proc}) {
+    foreach my $pid (keys %{$proc}) {
         my $proc = $proc->{$pid};
         while ( my ($key, $value) = each %{$filter} ) {
-            if (exists $proc->{$key}) {
-                if (ref $proc->{$key} eq 'HASH') {
-                    foreach my $v (values %{$proc->{$key}}) {
-                        if ($self->_compare($v, $value)) {
-                            push @hits, $pid;
-                            next PID;
-                        }
-                    }
-                } elsif ($self->_compare($proc->{$key}, $value)) {
-                    push @hits, $pid;
-                    next PID;
-                }
+            if (exists $proc->{$key} && $self->_compare($proc->{$key}, $value)) {
+                push @hits, $pid;
             }
         }
     }
@@ -364,8 +398,7 @@ sub pstop {
 #
 
 sub _compare {
-    my ($self, $x, $y) = @_;
-
+    my ($class, $x, $y) = @_;
     if (ref($y) eq 'Regexp') {
         return $x =~ $y;
     } elsif ($y =~ s/^eq://) {
@@ -377,9 +410,8 @@ sub _compare {
     } elsif ($y =~ s/^lt://) {
         return $x < $y;
     } else {
-        croak ref($self).": bad search() / psfind() operator '$y'";
+        croak "$class: bad search() / psfind() operator '$y'";
     }
-
     return undef;
 }
 
